@@ -22,45 +22,45 @@ SUTTON_CHEN_PARAMS = {
     'Al': {'n': 7,  'm': 6, 'epsilon': 1.0, 'af': 4.05, 'c': 16.399}
 }
 #------------------------------------------------------------------------------------------
-@jit(nopython=True)
+@jit(nopython=True, fastmath=True)
 def opt_sc_metal(positions, n, m, epsilon, af, c):
-    """Calcula energía Sutton-Chen (optimizada con Numba)"""
+    """Calculate Sutton-Chen energy (optimized with Numba)"""
     N = len(positions)
+    s1 = np.zeros(N)
+    s2 = np.zeros(N)
     energy = 0.0
-    for i in range(N):
-        s1i = 0.0
-        s2i = 0.0
-        for j in range(N):
-            if i != j:
-                rij = positions[i] - positions[j]
-                r = np.linalg.norm(rij)
-                if r < cutoff:
-                    var0 = af / r
-                    s1i += var0 ** n
-                    s2i += var0 ** m
-        sqrtrho = np.sqrt(s2i)
-        ei = epsilon * (0.5 * s1i - c * sqrtrho)
-        energy += ei
+    for i in range(N - 1):
+        for j in range(i + 1, N):
+            rij = positions[i] - positions[j]
+            r = np.linalg.norm(rij)
+            if r < cutoff:
+                invr = af / r
+                var0n = invr ** n
+                var0m = invr ** m
+                s1[i] += var0n
+                s1[j] += var0n
+                s2[i] += var0m
+                s2[j] += var0m
+    energy = epsilon * (0.5 * np.sum(s1) - c * np.sum(np.sqrt(s2)))
     return energy
 #------------------------------------------------------------------------------------------
-@jit(nopython=True)
+#https://wanglab.hosted.uark.edu/DLPOLY2/node56.html
+@jit(nopython=True, fastmath=True)
 def opt_sc_forces(positions, n, m, epsilon, af, c):
     N = len(positions)
     forces = np.zeros_like(positions)
-
-    s2i_array = np.zeros(N)
-    for i in range(N):
-        s2i = 0.0
-        for j in range(N):
-            if i != j:
-                rij = positions[i] - positions[j]
-                r = np.linalg.norm(rij)
-                if r < cutoff:
-                    var0 = af / r
-                    s2i += var0 ** m
-        s2i_array[i] = s2i
-    
-    for i in range(N):
+    s2 = np.zeros(N)
+    for i in range(N - 1):
+        for j in range(i + 1, N):
+            rij = positions[i] - positions[j]
+            r = np.linalg.norm(rij)
+            if r < cutoff:
+                var0 = af / r
+                var0_m = var0 ** m
+                s2[i] += var0_m
+                s2[j] += var0_m
+    sqrt_rho = np.sqrt(s2)
+    for i in range(N - 1):
         for j in range(i + 1, N):
             rij = positions[i] - positions[j]
             r = np.linalg.norm(rij)
@@ -69,11 +69,9 @@ def opt_sc_forces(positions, n, m, epsilon, af, c):
                 var0_n = var0 ** n
                 var0_m = var0 ** m
                 f_rep = epsilon * n * var0_n / r
-                sqrt_rho_i = np.sqrt(s2i_array[i])
-                sqrt_rho_j = np.sqrt(s2i_array[j])
-                f_dens_i = epsilon * c * m * var0_m / (2.0 * sqrt_rho_i * r)
-                f_dens_j = epsilon * c * m * var0_m / (2.0 * sqrt_rho_j * r)
-                f_scalar = f_rep + f_dens_i + f_dens_j
+                f_dens_i = epsilon * c * m * var0_m / (2.0 * sqrt_rho[i] * r)
+                f_dens_j = epsilon * c * m * var0_m / (2.0 * sqrt_rho[j] * r)
+                f_scalar = f_rep - f_dens_i - f_dens_j
                 fij = f_scalar * (rij / r)
                 forces[i] += fij
                 forces[j] -= fij
@@ -88,8 +86,7 @@ def sutton_chen_energy(atoms, metal_type):
         params['m'], 
         params['epsilon'], 
         params['af'], 
-        params['c'], 
-        cutoff
+        params['c']
     )
 #------------------------------------------------------------------------------------------
 def sutton_chen_forces(atoms, metal_type):
@@ -102,8 +99,7 @@ def sutton_chen_forces(atoms, metal_type):
         params['m'], 
         params['epsilon'], 
         params['af'], 
-        params['c'], 
-        cutoff
+        params['c']
     )
 #------------------------------------------------------------------------------------------
 def opt_sc(atoms, metal_type='Cu'):
@@ -113,10 +109,8 @@ def opt_sc(atoms, metal_type='Cu'):
     m = params['m']
     epsilon = params['epsilon']
     c = params['c']
-    
     cluster = atoms.copy()
     x0 = cluster.get_positions()
-    
     def objective(x):
         positions = x.reshape(-1, 3)
         return opt_sc_metal(positions, n, m, epsilon, af, c)
@@ -124,7 +118,6 @@ def opt_sc(atoms, metal_type='Cu'):
         positions = x.reshape(-1, 3)
         forces = opt_sc_forces(positions, n, m, epsilon, af, c)
         return -forces.reshape(-1)  # Gradiente = -Fuerzas
-    
     result = minimize(
         fun=objective,
         x0=x0.reshape(-1),
@@ -132,17 +125,15 @@ def opt_sc(atoms, metal_type='Cu'):
         method=method,
         options={'disp': False, 'gtol': gtol}
     )
-    
     opt_positions = result.x.reshape(-1, 3)
     energy_opt = result.fun
-    
     cluster_opt = atoms.copy()
     cluster_opt.set_positions(opt_positions)
     cluster_opt.info['e'] = energy_opt
-    cluster_opt.info['c'] = 1
+    cluster_opt.info['c'] = 1 if result.success else 0
     return cluster_opt
 #------------------------------------------------------------------------------------------
-def parallel_opt_SC(mol_list, metal_type='Cu', n_jobs=-1):
+def opt_SC_parallel(mol_list, metal_type='Cu', n_jobs=-1):
     start_time = time.time()
     n1=len(mol_list)
     if not isinstance(mol_list, list): mol_list = [mol_list]
@@ -152,15 +143,15 @@ def parallel_opt_SC(mol_list, metal_type='Cu', n_jobs=-1):
     print("Local OPT parallel at %.2f s [%d -> %d]" % (end_time-start_time, n1, n2))
     return results
 #------------------------------------------------------------------------------------------
-#if __name__ == "__main__":
-#    from aegon.libutils import readxyzs, writexyzs
-#    scnet = readxyzs('SuttonChen003to080Cu.xyz')
-#    for imol in scnet:
-#        n=len(imol)
-#        energy_true= imol.info['e']
-#        imol=opt_sc(imol, metal_type='Cu')
-#        energy_sc= imol.info['e']
-#        print("#%s E_x = %f E_T= %f" %(str(n).zfill(3), energy_sc, energy_true))
-#    #print('Parallel optimization of all clusters')
-#    #all_optimized = parallel_opt_SC(scnet, metal_type='Cu', n_jobs=14)
-#    #writexyzs(all_optimized, 'all_optimized_clusters.xyz')
+#from aegon.libutils import readxyzs
+#listmol=readxyzs('../SC052.xyz')
+#start_time = time.time()
+#for imol in listmol: e=sutton_chen_energy(imol, metal_type='Pd')
+##print(e)
+#end_time = time.time()
+#print("Referencia at %.3f s" % (end_time-start_time))
+#start_time = time.time()
+#for imol in listmol: e=sutton_chen_forces(imol, metal_type='Pd')
+##print(e)
+#end_time = time.time()
+#print("Propuesta at %.3f s" % (end_time-start_time))
